@@ -1080,13 +1080,13 @@ void subst_chars(gchar *str, gchar *orig, gchar subst)
 
 void subst_for_filename(gchar *str)
 {
-	subst_chars(str, " \t\r\n\"/\\", '_');
+	subst_chars(str, "\t\r\n\\/*", '_');
 }
 
 void subst_for_shellsafe_filename(gchar *str)
 {
 	subst_for_filename(str);
-	subst_chars(str, "|&;()<>'!{}[]",'_');
+	subst_chars(str, " \"'|&;()<>'!{}[]",'_');
 }
 
 gboolean is_header_line(const gchar *str)
@@ -2020,7 +2020,7 @@ gint remove_numbered_files(const gchar *dir, guint first, guint last)
 	DIR *dp;
 	struct dirent *d;
 	gchar *prev_dir;
-	gint fileno;
+	gint file_no;
 
 	prev_dir = g_get_current_dir();
 
@@ -2037,8 +2037,8 @@ gint remove_numbered_files(const gchar *dir, guint first, guint last)
 	}
 
 	while ((d = readdir(dp)) != NULL) {
-		fileno = to_number(d->d_name);
-		if (fileno >= 0 && first <= fileno && fileno <= last) {
+		file_no = to_number(d->d_name);
+		if (file_no > 0 && first <= file_no && file_no <= last) {
 			if (is_dir_exist(d->d_name))
 				continue;
 			if (unlink(d->d_name) < 0)
@@ -2064,7 +2064,7 @@ gint remove_numbered_files_not_in_list(const gchar *dir, GSList *numberlist)
 	DIR *dp;
 	struct dirent *d;
 	gchar *prev_dir;
-	gint fileno;
+	gint file_no;
 
 	prev_dir = g_get_current_dir();
 
@@ -2081,9 +2081,9 @@ gint remove_numbered_files_not_in_list(const gchar *dir, GSList *numberlist)
 	}
 
 	while ((d = readdir(dp)) != NULL) {
-		fileno = to_number(d->d_name);
-		if (fileno >= 0 && (g_slist_find(numberlist, GINT_TO_POINTER(fileno)) == NULL)) {
-			debug_print("removing unwanted file %d from %s\n", fileno, dir);
+		file_no = to_number(d->d_name);
+		if (file_no > 0 && (g_slist_find(numberlist, GINT_TO_POINTER(file_no)) == NULL)) {
+			debug_print("removing unwanted file %d from %s\n", file_no, dir);
 			if (is_dir_exist(d->d_name))
 				continue;
 			if (unlink(d->d_name) < 0)
@@ -2115,7 +2115,7 @@ gint remove_expired_files(const gchar *dir, guint hours)
 	struct dirent *d;
 	struct stat s;
 	gchar *prev_dir;
-	gint fileno;
+	gint file_no;
 	time_t mtime, now, expire_time;
 
 	prev_dir = g_get_current_dir();
@@ -2136,8 +2136,8 @@ gint remove_expired_files(const gchar *dir, guint hours)
 	expire_time = hours * 60 * 60;
 
 	while ((d = readdir(dp)) != NULL) {
-		fileno = to_number(d->d_name);
-		if (fileno >= 0) {
+		file_no = to_number(d->d_name);
+		if (file_no > 0) {
 			if (stat(d->d_name, &s) < 0) {
 				FILE_OP_ERROR(d->d_name, "stat");
 				continue;
