@@ -160,6 +160,9 @@ static void import_mbox_cb	 (MainWindow	*mainwin,
 static void export_mbox_cb	 (MainWindow	*mainwin,
 				  guint		 action,
 				  GtkWidget	*widget);
+static void export_list_mbox_cb  (MainWindow 	*mainwin, 
+				  guint 	 action,
+				  GtkWidget 	*widget);
 static void empty_trash_cb	 (MainWindow	*mainwin,
 				  guint		 action,
 				  GtkWidget	*widget);
@@ -444,6 +447,8 @@ static GtkItemFactoryEntry mainwin_entries[] =
 	{N_("/_File/---"),			NULL, NULL, 0, "<Separator>"},
 	{N_("/_File/_Import mbox file..."),	NULL, import_mbox_cb, 0, NULL},
 	{N_("/_File/_Export to mbox file..."),	NULL, export_mbox_cb, 0, NULL},
+	{N_("/_File/_Export selected to mbox file..."),	
+						NULL, export_list_mbox_cb, 0, NULL},
 	{N_("/_File/---"),			NULL, NULL, 0, "<Separator>"},
 	{N_("/_File/Empty all _Trash folders"),	"<shift>D", empty_trash_cb, 0, NULL},
 	{N_("/_File/---"),			NULL, NULL, 0, "<Separator>"},
@@ -1105,6 +1110,7 @@ void main_window_destroy(MainWindow *mainwin)
 {
 	/* TODO : destroy other component */
 	messageview_destroy(mainwin->messageview);
+	mainwin->messageview = NULL;
 }
 
 void main_window_update_actions_menu(MainWindow *mainwin)
@@ -1474,6 +1480,12 @@ void main_window_get_size(MainWindow *mainwin)
 {
 	GtkAllocation *allocation;
 
+	if (mainwin->messageview == NULL) {
+		debug_print("called after messageview "
+			    "has been deallocated!\n");
+		return;
+	}
+
 	allocation = &(GTK_WIDGET_PTR(mainwin->summaryview)->allocation);
 
 	if (allocation->width > 1 && allocation->height > 1) {
@@ -1980,6 +1992,7 @@ static void main_window_set_widgets(MainWindow *mainwin, SeparateType type)
 		if (prefs_common.folderview_visible)
 			gtk_widget_show(folderwin);
 	}
+
 	if (type & SEPARATE_MESSAGE) {
 		messagewin = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 		gtk_window_set_title(GTK_WINDOW(messagewin),
@@ -1997,6 +2010,9 @@ static void main_window_set_widgets(MainWindow *mainwin, SeparateType type)
 				   mainwin);
 		if (messageview_is_visible(mainwin->messageview))
 			gtk_widget_show(messagewin);
+	} else {
+		mainwin->messageview->statusbar = mainwin->statusbar;
+		mainwin->messageview->statusbar_cid = mainwin->messageview_cid;
 	}
 
 	gtk_widget_set_usize(GTK_WIDGET_PTR(mainwin->folderview),
@@ -2283,6 +2299,9 @@ static gint message_window_close_cb(GtkWidget *widget, GdkEventAny *event,
 		(mainwin->menu_factory, "/View/Show or hide/Message view");
 	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menuitem), FALSE);
 
+	mainwin->messageview->statusbar = mainwin->statusbar;
+	mainwin->messageview->statusbar_cid = mainwin->messageview_cid;
+
 	return TRUE;
 }
 
@@ -2344,6 +2363,12 @@ static void export_mbox_cb(MainWindow *mainwin, guint action,
 	export_mbox(mainwin->summaryview->folder_item);
 }
 
+static void export_list_mbox_cb(MainWindow *mainwin, guint action,
+				GtkWidget *widget)
+{
+	summaryview_export_mbox_list(mainwin->summaryview);
+}
+
 static void empty_trash_cb(MainWindow *mainwin, guint action,
 			   GtkWidget *widget)
 {
@@ -2437,16 +2462,12 @@ void main_window_reply_cb(MainWindow *mainwin, guint action,
 {
 	MessageView *msgview = (MessageView*)mainwin->messageview;
 	GSList *msginfo_list = NULL;
-	gchar *body;
 
 	g_return_if_fail(msgview != NULL);
 
 	msginfo_list = summary_get_selection(mainwin->summaryview);
 	g_return_if_fail(msginfo_list != NULL);
-	
-	body = messageview_get_selection(msgview);
-	compose_reply_mode((ComposeMode)action, msginfo_list, body);
-	g_free(body);
+	compose_reply_from_messageview(msgview, msginfo_list, action);
 	g_slist_free(msginfo_list);
 }
 
