@@ -302,7 +302,7 @@ static GtkItemFactoryEntry msgview_entries[] =
 					NULL, create_filter_cb, FILTER_BY_TO, NULL},
 	{N_("/_Tools/_Create filter rule/by _Subject"),
 					NULL, create_filter_cb, FILTER_BY_SUBJECT, NULL},
-	{N_("/_Tools/Create processing rule/"),
+	{N_("/_Tools/Create processing rule"),
 					NULL, NULL, 0, "<Branch>"},
 	{N_("/_Tools/Create processing rule/_Automatically"),
 					NULL, create_processing_cb, FILTER_BY_AUTO, NULL},
@@ -668,6 +668,12 @@ static gint disposition_notification_send(MsgInfo *msginfo)
 		return -1;
 	}
 	
+	if (prefs_common.work_offline)
+		if (alertpanel(_("Offline warning"), 
+			       _("You're working offline. Override?"),
+			       _("Yes"), _("No"), NULL) != G_ALERTDEFAULT)
+			return 0;
+
 	/* send it */
 	path = folder_item_fetch_msg(queue, num);
 	ok = procmsg_send_message_queue(path);
@@ -736,6 +742,8 @@ gint messageview_show(MessageView *messageview, MsgInfo *msginfo,
 	if (messageview->msginfo != msginfo) {
 		procmsg_msginfo_free(messageview->msginfo);
 		messageview->msginfo = procmsg_msginfo_get_full_info(msginfo);
+		if (!messageview->msginfo)
+			messageview->msginfo = procmsg_msginfo_copy(msginfo);
 	}
 	headerview_show(messageview->headerview, messageview->msginfo);
 
@@ -755,6 +763,13 @@ gint messageview_show(MessageView *messageview, MsgInfo *msginfo,
 	else 
 		noticeview_hide(messageview->noticeview);
 
+	mimeinfo = procmime_mimeinfo_next(mimeinfo);
+	if (mimeinfo && (mimeinfo->type != MIMETYPE_TEXT || 
+	    (strcmp(mimeinfo->subtype, "plain") &&
+	     strcmp(mimeinfo->subtype, "html")))) {
+		mimeview_show_part(messageview->mimeview,mimeinfo);
+	}
+
 	g_free(file);
 
 	return 0;
@@ -1361,94 +1376,14 @@ static void compose_cb(gpointer data, guint action, GtkWidget *widget)
 static void reply_cb(gpointer data, guint action, GtkWidget *widget)
 {
 	MessageView *messageview = (MessageView *)data;
-	GSList *mlist = NULL;
+	GSList *msginfo_list = NULL;
 	MsgInfo *msginfo;
-	gchar *text = NULL;
-	ComposeMode mode = (ComposeMode)action;
-	TextView *textview;
 
-	msginfo = messageview->msginfo;
-	mlist = g_slist_append(NULL, msginfo);
+	g_return_if_fail(messageview->msginfo);
 
-	textview = messageview_get_current_textview(messageview);
-	text = gtkut_editable_get_selection
-		(GTK_EDITABLE(textview->text));
-	if (text && *text == '\0') {
-		g_free(text);
-		text = NULL;
-	}
-
-	switch (mode) {
-	case COMPOSE_REPLY:
-		compose_reply(msginfo, prefs_common.reply_with_quote,
-		    	      FALSE, prefs_common.default_reply_list, FALSE, text);
-		break;
-	case COMPOSE_REPLY_WITH_QUOTE:
-		compose_reply(msginfo, TRUE, FALSE, prefs_common.default_reply_list, FALSE, text);
-		break;
-	case COMPOSE_REPLY_WITHOUT_QUOTE:
-		compose_reply(msginfo, FALSE, FALSE, prefs_common.default_reply_list, FALSE, NULL);
-		break;
-	case COMPOSE_REPLY_TO_SENDER:
-		compose_reply(msginfo, prefs_common.reply_with_quote,
-			      FALSE, FALSE, TRUE, text);
-		break;
-	case COMPOSE_FOLLOWUP_AND_REPLY_TO:
-		compose_followup_and_reply_to(msginfo,
-					      prefs_common.reply_with_quote,
-					      FALSE, FALSE, text);
-		break;
-	case COMPOSE_REPLY_TO_SENDER_WITH_QUOTE:
-		compose_reply(msginfo, TRUE, FALSE, FALSE, TRUE, text);
-		break;
-	case COMPOSE_REPLY_TO_SENDER_WITHOUT_QUOTE:
-		compose_reply(msginfo, FALSE, FALSE, FALSE, TRUE, NULL);
-		break;
-	case COMPOSE_REPLY_TO_ALL:
-		compose_reply(msginfo, prefs_common.reply_with_quote,
-			      TRUE, FALSE, FALSE, text);
-		break;
-	case COMPOSE_REPLY_TO_ALL_WITH_QUOTE:
-		compose_reply(msginfo, TRUE, TRUE, FALSE, FALSE, text);
-		break;
-	case COMPOSE_REPLY_TO_ALL_WITHOUT_QUOTE:
-		compose_reply(msginfo, FALSE, TRUE, FALSE, FALSE, NULL);
-		break;
-	case COMPOSE_REPLY_TO_LIST:
-		compose_reply(msginfo, prefs_common.reply_with_quote,
-			      FALSE, TRUE, FALSE, text);
-		break;
-	case COMPOSE_REPLY_TO_LIST_WITH_QUOTE:
-		compose_reply(msginfo, TRUE, FALSE, TRUE, FALSE, text);
-		break;
-	case COMPOSE_REPLY_TO_LIST_WITHOUT_QUOTE:
-		compose_reply(msginfo, FALSE, FALSE, TRUE, FALSE, NULL);
-		break;
-	case COMPOSE_FORWARD:
-		if (prefs_common.forward_as_attachment) {
-			compose_reply_mode(COMPOSE_FORWARD_AS_ATTACH, mlist, text);
-			return;
-		} else {
-			compose_reply_mode(COMPOSE_FORWARD_INLINE, mlist, text);
-			return;
-		}
-		break;
-	case COMPOSE_FORWARD_INLINE:
-		compose_forward(NULL, msginfo, FALSE, text, FALSE);
-		break;
-	case COMPOSE_FORWARD_AS_ATTACH:
-		compose_forward_multiple(NULL, mlist);
-		break;
-	case COMPOSE_REDIRECT:
-		compose_redirect(NULL, msginfo);
-		break;
-	default:
-		g_warning("compose_reply(): invalid Compose Mode: %d\n", mode);
-	}
-
-	/* summary_set_marks_selected(summaryview); */
-	g_free(text);
-	g_slist_free(mlist);
+	msginfo_list = g_slist_append(msginfo_list, messageview->msginfo);
+	compose_reply_from_messageview(messageview, msginfo_list, action);
+	g_slist_free(msginfo_list);
 }
 
 static void reedit_cb(gpointer data, guint action, GtkWidget *widget)
@@ -1536,7 +1471,7 @@ void messageview_set_menu_sensitive(MessageView *messageview)
 	GtkItemFactory *ifactory;
 	GtkWidget *menuitem;
 
-	if (!messageview && !messageview->new_window) 
+	if (!messageview || !messageview->new_window) 
 		return;
 	/* do some smart things */
 	if (!messageview->menubar) return;
