@@ -658,6 +658,11 @@ static void destroy_widget_func(PrefsPage *_page)
 	gtk_container_remove(GTK_CONTAINER (page->vbox), notebook);
 }
 
+static gboolean can_close_func(PrefsPage *page_)
+{	
+	return prefs_account_apply() >= 0;
+}
+
 static void save_func(PrefsPage * _page)
 {
 	if (prefs_account_apply() >= 0)
@@ -676,6 +681,7 @@ void prefs_account_init()
         account_page.page.create_widget = create_widget_func;
         account_page.page.destroy_widget = destroy_widget_func;
         account_page.page.save_page = save_func;
+	account_page.page.can_close = can_close_func;
 
         prefs_account_register_page((PrefsPage *) &account_page);
 }
@@ -1584,6 +1590,7 @@ static void prefs_account_send_create(void)
 	optmenu_menu = gtk_menu_new ();
 
 	MENUITEM_ADD (optmenu_menu, menuitem, _("Automatic"), 0);
+	MENUITEM_ADD (optmenu_menu, menuitem, "PLAIN", SMTPAUTH_PLAIN);
 	MENUITEM_ADD (optmenu_menu, menuitem, "LOGIN", SMTPAUTH_LOGIN);
 	MENUITEM_ADD (optmenu_menu, menuitem, "CRAM-MD5", SMTPAUTH_CRAM_MD5);
 	MENUITEM_ADD (optmenu_menu, menuitem, "DIGEST-MD5", SMTPAUTH_DIGEST_MD5);
